;
; manusr-gen.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; manusr-gen [files]
;;
;;   Reads plugin configuration from seq files, and outputs the
;;   compiled manager configuration file. As
;;   the output is binary it's normally redirected to a file.
;;
;;   Commonly used in the plugin registry directory as:
;;
;;   #manusr-gen * >@//:man,usr
;;
;;   Commands in plugin configuration files:
;;
;;     path "1//plugins/:"
;;       set plugin directory
;;
;;     file "plugin"
;;       set actual plugin filename
;;
;;     key return
;;       set actual manager key (return/2/3/4)
;;
;;     prio number
;;       set actual priority (0/1/2/3/4)
;;
;;     match "pattern"
;;       pattern to match (can be more than one)
;;
;;   The "match" command generates new entries based on the actual
;;   values set by the previous commands.
;;
;;   When generating the output higher priority entries are grouped earlier
;;   in the list, thereby allowing specific matches over more generic ones.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= 1

tmp		= $2
mp              = $fd

banner		= "manusr-gen 0.1 by soci/singular"

		.include "lib.asm"

T_PATH          = 1
T_FILE          = 2
T_KEY           = 3
T_MATCH         = 4
T_PRIO          = 5
T_COMMENT       = 6
T_QUOT          = 7
T_RETURN        = 8
T_0             = 9
T_1             = 10
T_2             = 11
T_3             = 12
T_4             = 13

		.section code
start2		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1
                ldx #255
-               inx
                lda defpath,x
                sta path,x
                bne -
                lda #<mem
                sta mp
                lda #>mem
                sta mp+1

		.section data
defpath         .null "1//plugins/:"
		.send data

loop		ldy #0
		lda (p1),y
		bne +
q	        jsr outp
		jsr $ffcc
		jmp exit

+		ldx p1
		ldy p1+1
		pha
                jsr addstring
		pla
		inx
		bne +
		iny
+		jsr onefile
		ldx $91
		cpx #$7f
		beq q
		jmp loop

outp            ldy #0
                tya
                sta (mp),y
		ldx #3
                jsr $ffc9
                lda #32
                jsr $ffd2
                jsr $ffd2
                ldx #255
-               inx
		lda path,x
                jsr $ffd2
                cmp #0
                bne -
                lda #1
                jsr writelist
                lda #2
                jsr writelist
                lda #3
                jsr writelist
                lda #4
                jsr writelist
                lda #0
                jsr $ffd2
                jsr $ffcc
                jmp exit

writelist       .proc
                pha
                ora #$40
                jsr do
                pla
                pha
                ora #$30
                jsr do
                pla
                pha
                ora #$20
                jsr do
                pla
                pha
                ora #$10
                jsr do
                pla
                jsr do
		lda #"="
                jsr $ffd2
                lda #0
                jmp $ffd2

do              sta $02
		lda #<mem
                sta mp
                lda #>mem
                sta mp+1

                ldy #0
lp              lda (mp),y
                beq kesz
                tax
                iny
                lda (mp),y
                cmp $02
                bne nj
                dex
-               dex
                beq nj
                iny
		lda (mp),y
                jsr $ffd2
                jmp -

nj              ldy #0
                lda (mp),y
                clc
                adc mp
                sta mp
                bcc lp
                inc mp+1
                gne lp

kesz            rts
                .pend

onefile		jsr $ffbd
                lda #$20
                sta prio
		lda #1
		ldx mydrive
		ldy #0
                sty file
                sty key
		jsr $ffba
		jsr $ffc0

                ldx #1
                jsr $ffc6
next            ldx $90
                beq +
                jsr $ffcc
                lda #1
                jmp $ffc3
+

-               jsr $ffcf
                sta line,x
                cmp #13
                beq v
                lda #$bf
                bit $90
                bne v
                inx
                bvc -

v               lda #0
                sta line,x

                ldx #255
                jsr gettoken

                cmp #T_COMMENT
                beq next

                cmp #T_FILE     ;file
                bne nfile
                jsr gettoken
                cmp #T_QUOT      ;"
                bne err
                ldy #255
-               inx
                iny
                lda line,x
                beq err
                sta file,y
                eor #34
                bne -
                sta file,y
err             jmp next

nfile
                cmp #T_PATH      ;path
                bne npath
                jsr gettoken
                cmp #T_QUOT      ;"
                bne err
                ldy #255
-               inx
                iny
                lda line,x
                beq err
                sta path,y
                eor #34
                bne -
                sta path,y
                jmp next
npath
                cmp #T_KEY      ;key
                bne nkey
                jsr gettoken
                cmp #T_RETURN   ;return
                beq k1
                cmp #T_2        ;2
                beq k2
                cmp #T_3        ;3
                beq k3
                cmp #T_4        ;4
                bne err
                lda #$04
                .byte $2c
k3              lda #$03
                .byte $2c
k2              lda #$02
                .byte $2c
k1              lda #$01
                sta key
                jmp next

nkey
                cmp #T_PRIO     ;key
                bne nprio
                jsr gettoken
                cmp #T_0        ;0
                beq pr0
                cmp #T_1        ;1
                beq pr1
                cmp #T_2        ;2
                beq pr2
                cmp #T_3        ;3
                beq pr3
                cmp #T_4        ;4
                bne err2
                lda #$40
                .byte $2c
pr3             lda #$30
                .byte $2c
pr2             lda #$20
                .byte $2c
pr1             lda #$10
                .byte $2c
pr0             lda #$00
                sta prio
err2            jmp next

nprio
                cmp #T_MATCH  ;pattern
                bne err
lp              jsr gettoken
                cmp #T_QUOT      ;"
                bne err2
                ldy #255
-               inx
                iny
                lda line,x
                beq err2
                sta pattern,y
                eor #34
                bne -
                sta pattern,y
                txa
                pha

                ldy #1
                lda key
                ora prio
                sta (mp),y
                ldx #255
-               iny
                inx
                lda file,x
                sta (mp),y
                bne -
                ldx #255
-               iny
                inx
                lda pattern,x
                sta (mp),y
                bne -
                iny
                tya
                ldy #0
                sta (mp),y
                clc
                adc mp
                sta mp
                bcc +
                inc mp+1
+
                pla
                tax
                jmp lp

gettoken        .proc
-               inx
                lda line,x
                cmp #32
                beq -

                ldy #0
                sty tmp
                stx ol+1
                .byte $2c
-               inx
                iny
nx              lda keys,y
                eor line,x
                beq -
                inc tmp
                asl
                beq kesz
ol              ldx #0

-               iny
                lda keys-1,y
                bpl -
                lda keys,y
                bne nx
                rts

kesz            lda tmp
                rts

		.section data
keys            .shift "path"	;1
		.shift "file"   ;2
                .shift "key"    ;3
                .shift "match"  ;4
                .shift "prio"   ;5
                .shift ";"      ;6
                .shift 34       ;7
                .shift "return" ;8
                .shift "0"      ;9
                .shift "1"      ;10
                .shift "2"      ;11
                .shift "3"      ;12
                .shift "4"      ;13
                .byte 0
		.send data
		.pend
		.send code

		.section bss
path            .fill 256
file            .fill 20
pattern		.fill 32
key             .fill 1
prio            .fill 1

mem             .fill 8192

line            .fill 256
		.send bss

